package cn.estsh.i3plus.ext.mes.apiservice.controller.busi;

import cn.estsh.i3plus.ext.mes.api.base.IMesCcscTaskService;
import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.busi.IMaximoService;
import cn.estsh.i3plus.ext.mes.pojo.model.MaximoParamModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesShift;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShiftRepository;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/white/maximo")
@Api(description = "白名单接口maximo")
public class MesWhiteListController extends MesBaseController {

    @Autowired
    private IMaximoService maximoService;

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @Autowired
    private IMesCcscTaskService mesCcscTaskService;
    @Autowired
    private IMesWorkOrderService workOrderService;

    @Autowired
    private MesProductionRecordRepository productionRecordRao;
    @Autowired
    private MesShiftRepository mesShiftRepository;
    @Autowired
    private IMesConfigService configService;
    @PostMapping(value = "/meter/main-data")
    @ApiOperation(value = "Maximo仪表主数据", notes = "Maximo仪表主数据")
    public Map doMeterMainData(@RequestBody MaximoParamModel paramModel) {
        // 数据校验
        ValidatorBean.checkNotNull(paramModel.getMeterName(), "meterName不能为空");
        ValidatorBean.checkNotNull(paramModel.getAssetNum(), "assetNum不能为空");
        ValidatorBean.checkNotNull(paramModel.getBaseMeasureUnitId(), "baseMeasureUnitId不能为空");
        ValidatorBean.checkNotNull(paramModel.getSiteId(), "siteId不能为空");
        Map resultMap =  maximoService.doMeterMainData(paramModel);

        LOGGER.info("Maximo仪表主数据 返回 resultMap:{}", resultMap);

        return resultMap;

    }

    @PostMapping(value = "/asset/main-data")
    @ApiOperation(value = "Maximo设备主数据", notes = "Maximo设备主数据")
    public Map doAssetMainData(@RequestBody MaximoParamModel paramModel) {
        // 数据校验
        ValidatorBean.checkNotNull(paramModel.getAssetNum(), "assetNum不能为空");
        ValidatorBean.checkNotNull(paramModel.getDescription(), "description不能为空");
        ValidatorBean.checkNotNull(paramModel.getSiteId(), "siteId不能为空");
        ValidatorBean.checkNotNull(paramModel.getStatus(), "status不能为空");
        ValidatorBean.checkNotNull(paramModel.getFailureCode(), "failureCode不能为空");
        Map resultMap =  maximoService.doAssetMainData(paramModel);

        LOGGER.info("Maximo设备主数据 返回 resultMap:{}", resultMap);

        return resultMap;
//        return ResultBean.success().setResultMap(resultMap).setSuccess((Boolean) resultMap.get("success"));

    }
    @GetMapping(value = "/ccscTest")
    @ApiOperation(value = "ccsc", notes = " ")
    public ResultBean doCcscTest(String organizeCode) {
        List<MesWorkOrder> mesWorkOrderList = mesWorkOrderService.queryMesWorkOrderProcessToSap(organizeCode);
        if (!CollectionUtils.isEmpty(mesWorkOrderList)){
            mesCcscTaskService.doCcscTaskByMesWorkOrder(organizeCode,mesWorkOrderList);
        }
         return ResultBean.success().setSuccess(true);
    }
    @GetMapping("/reportWorkByPreDayJob")
    @ApiOperation(value = "job测试")
    public ResultBean queryReworkTaskByPager(String organizeCode) {
        try {
            //遍历系统参数指定的产线获取对应班次，找到早班的开始时间，
            String workCenterCodes = configService.getCfgValue(organizeCode, "MES_PCN_WORK_CENTER_NO");
            List<String> workCenterCodeList = Arrays.asList(workCenterCodes.split(","));
            String userName = "REPORT_PRE_DAY_JOB";

            for (String workCenterCode : workCenterCodeList) {
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", packBean);
                List<MesShift> mesShiftList = mesShiftRepository.findByHqlWhere(packBean);
                 //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
                List<MesProductionRecord> mesProductionRecordList = workOrderService.getPreDayReportMesProductionRecord(organizeCode,mesShiftList,workCenterCode);
                if(CollectionUtils.isEmpty(mesProductionRecordList)){
                    continue;
                }
                //2. 查询 mesProduceSn
                List<MesProduceSn> unReportMesProduceSn = workOrderService.getPreDayReportMesProduceSn(organizeCode,mesProductionRecordList);
                if(CollectionUtils.isEmpty(unReportMesProduceSn)){
                    continue;
                }
                List<MesWorkOrder> mesWorkOrders = workOrderService.insertMesWorkOrder(unReportMesProduceSn, mesShiftList,organizeCode, userName);
                if(CollectionUtils.isEmpty(mesWorkOrders)){
                    continue;
                }
                //根据产线+物料产生的工单报工
                for (MesWorkOrder mesWorkOrder : mesWorkOrders) {
                    List<MesProduceSn> mesProduceSnList = unReportMesProduceSn.stream().filter(w -> w.getPartNo().equalsIgnoreCase(mesWorkOrder.getPartNo())).collect(Collectors.toList());
                    workOrderService.doPcnJobProductReport(mesProduceSnList,mesShiftList,organizeCode,userName, mesWorkOrder);
                }
                for (MesProductionRecord mesProductionRecord : mesProductionRecordList) {
                    mesProductionRecord.setReportStatus(20);
                    ConvertBean.serviceModelUpdate(mesProductionRecord, userName);
                }
                productionRecordRao.saveAll(mesProductionRecordList);
            }
            return ResultBean.success("查询成功").setResultList(null);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
